/* part of the shansyn spherical harmonics package, see COPYRIGHT for license */
/* $Id: trig_constants.h,v 1.5 2001/04/26 21:09:56 becker Exp becker $ */
#ifndef PI
#define PI 3.14159265358979324
#endif
#ifndef TWOPI
#define TWOPI 6.28318530717958647
#endif
#ifndef TWO_PI
#define TWO_PI TWOPI
#endif
#ifndef PIOVERONEEIGHTY
#define PIOVERONEEIGHTY 0.0174532925199433
#endif
#ifndef PIHALF
#define PIHALF 1.57079632679489661
#endif
#ifndef SQRT_TWO
#define SQRT_TWO 1.41421356237309504
#endif
#ifndef  TWO_PISQR
#define TWO_PISQR 19.73920880217871723
#endif
#ifndef ONEEIGHTYOVERPI 
#define ONEEIGHTYOVERPI  57.2957795130823
#endif
#ifndef HALF_SQRT_TWO 
#define HALF_SQRT_TWO    0.70710678118654
#endif
#ifndef TWO_SQRT_PI
#define TWO_SQRT_PI 3.5449077018110321
#endif
#ifndef ONE_OVER_SQRT_FOUR_PI
#define ONE_OVER_SQRT_FOUR_PI 0.282094791773878
#endif
#ifndef SQRT_TWO_PI
#define SQRT_TWO_PI 2.5066282746310005
#endif
#ifndef SQRT_THREE_FOURTHS
#define SQRT_THREE_FOURTHS 0.8660254037844386
#endif
#ifndef SQRT_THREE
#define SQRT_THREE 1.732050807568877
#endif
