/* part of the shansyn spherical harmonics package, see COPYRIGHT for license */
/* $Id: legendre_macros.h,v 1.3 2001/03/08 03:04:49 becker Exp $ */
#ifndef POSLM
 #define POSLM(l, m) ( (((l) * (1+(l))) / 2) + (m))
#endif
#ifndef LPOSLM
 #define LPOSLM(l, m, j) (((j)*(lmsize)+POSLM(l,m)))
#endif
#ifndef P
 #define P(l, m, j)        (*(p       +LPOSLM(l, m, j)))
#endif
#ifndef DPTHETA
 #define DPTHETA(l, m, j)  (*(dptheta +LPOSLM(l, m, j)))
#endif
#ifndef DP2THETA
 #define DP2THETA(l, m, j) (*(dp2theta+LPOSLM(l, m, j)))
#endif
