#include <stdlib.h>
#include <stdio.h>

void mycalloc_cp(COMP_PRECISION **,int );
void mycalloc_dp(DATA_PRECISION **,int );
void mymalloc_dp(DATA_PRECISION **,int );
void mymalloc_cp(COMP_PRECISION **,int );
void myrealloc_dp(DATA_PRECISION **,int );
void myrealloc_cp(COMP_PRECISION **,int );
void zero_cp(COMP_PRECISION *,int );
void zero_dp(DATA_PRECISION *,int );
