#
# part of shansyn, included from the makefile
#
# $Id: machine_dependent.m,v 1.7 2001-07-26 12:57:16-07 becker Exp tbecker $
#
# compiler flags, change for your machine
# 
# you can either set them here or in a .cshrc or similar as an 
#    environment variable
# you will also have to set an environment variable 'ARCH' to indicate
# the hardward platform you are compiling for
# 
#
# main GMT directory, we will need some objects/libraries  from 
# $(GMT_DIR)/src/
#
GMT_DIR = $(GMTHOME)
GMT_LIB_DIR = $(GMTHOME)/lib/
#
# netcdf directory, should be set as environment variable
#
#NETCDFDIR = /usr/local/src/netcdf-3.6.0-p1/
NETCDF_DIR = $(NETCDFHOME)/

LDFLAGS = -L/usr/lib/gcc/x86_64-redhat-linux/4.1.2/ -lgfortran -lm
#
# define these is shana is to use LAPACK
#
LAPACK_DEFINE = -DUSE_LAPACK
LAPACK_LOAD = $(MATHLIBS)
