/* part of the shansyn spherical harmonics package, see COPYRIGHT for license */
/* $Id: spear.h,v 1.2 2001/03/08 03:04:49 becker Exp $ */
/* 
   spear.c  related numerical recipes routines

*/
void spear(COMP_PRECISION *data1, COMP_PRECISION *data2, unsigned long n, COMP_PRECISION *d, COMP_PRECISION *zd, COMP_PRECISION *probd, COMP_PRECISION *rs, COMP_PRECISION *probrs);
COMP_PRECISION betai(COMP_PRECISION a, COMP_PRECISION b, COMP_PRECISION x);
void crank(unsigned long n, COMP_PRECISION *w, COMP_PRECISION *s);
COMP_PRECISION gammln(COMP_PRECISION xx);
COMP_PRECISION betacf(COMP_PRECISION a, COMP_PRECISION b, COMP_PRECISION x);
void sort2(unsigned long n, COMP_PRECISION *arr, COMP_PRECISION *brr);
void free_ivector(int *v, long nl, long nh);
int *ivector(long nl, long nh);
COMP_PRECISION *vector(long nl, long nh);
void free_vector(COMP_PRECISION *v, long nl, long nh);
void nrerror(char *text);
COMP_PRECISION erfcc(COMP_PRECISION );

