/* part of the shansyn spherical harmonics package, see COPYRIGHT for license */
/* $Id: myio.h,v 1.2 2001/03/08 03:04:49 becker Exp $ */
#include <stdio.h>
#include <stdlib.h>
FILE *myopen(char *,char *);
FILE *myopen_wn(char *,char *, char *);
