/* part of the shansyn spherical harmonics package, see COPYRIGHT for license */
/* $Id: function_macros.h,v 1.12 2001/09/19 17:23:16 becker Exp becker $ */
#ifndef SQUARE	
#define SQUARE(x) ((x)*(x))
#endif

#define SIGNIFICANTLY_DIFFERENT(x, y) ((fabs((x)-(y))>1.0e-04)?(1):(0))

// the upper case BOOLEAN gets defined by GMT as int, so 
typedef int boolean;
#define TRUE 1
#define FALSE 0
#ifndef MAX
#define MAX(x, y) (((x)<(y))?(y):(x))
#endif 
#ifndef MIN
#define MIN(x, y) (((x)<(y))?(x):(y))
#endif 


#ifndef SIGN
#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))
#endif
#ifndef FMAX
#define FMAX MAX
#endif

#define ONE_MEGABYTE 1048576.0

